@@ -27,8 +27,9 @@ class SpMat : public SpBase< eT, SpMat<eT> >
   typedef eT                                elem_type;  //!< the type of elements stored in the matrix
   typedef typename get_pod_type<eT>::result  pod_type;  //!< if eT is std::complex<T>, pod_type is T; otherwise pod_type is eT
   
-  static const bool is_row = false;
-  static const bool is_col = false;
+  static const bool is_row  = false;
+  static const bool is_col  = false;
+  static const bool is_xvec = false;
   
   const uword n_rows;    //!< number of rows             (read-only)
   const uword n_cols;    //!< number of columns          (read-only)
@@ -131,6 +132,16 @@ class SpMat : public SpBase< eT, SpMat<eT> >
   template<typename T1> inline SpMat& operator/=(const Base<eT, T1>& m);
   template<typename T1> inline SpMat& operator%=(const Base<eT, T1>& m);
   
+  template<typename T1> inline explicit    SpMat(const Op<T1, op_diagmat>& expr);
+  template<typename T1> inline SpMat&  operator=(const Op<T1, op_diagmat>& expr);
+  template<typename T1> inline SpMat& operator+=(const Op<T1, op_diagmat>& expr);
+  template<typename T1> inline SpMat& operator-=(const Op<T1, op_diagmat>& expr);
+  template<typename T1> inline SpMat& operator*=(const Op<T1, op_diagmat>& expr);
+  template<typename T1> inline SpMat& operator/=(const Op<T1, op_diagmat>& expr);
+  template<typename T1> inline SpMat& operator%=(const Op<T1, op_diagmat>& expr);
+
+  //! explicit specification of sparse +/- scalar
+  template<typename T1, typename op_type> inline explicit SpMat(const SpToDOp<T1, op_type>& expr);
   
   //! construction of complex matrix out of two non-complex matrices
   template<typename T1, typename T2>
@@ -179,6 +190,15 @@ class SpMat : public SpBase< eT, SpMat<eT> >
   template<typename T1, typename spop_type> inline SpMat& operator%=(const mtSpOp<eT, T1, spop_type>& X);
   template<typename T1, typename spop_type> inline SpMat& operator/=(const mtSpOp<eT, T1, spop_type>& X);
   
+  // delayed mixed-type binary ops
+  template<typename T1, typename T2, typename spglue_type> inline             SpMat(const mtSpGlue<eT, T1, T2, spglue_type>& X);
+  template<typename T1, typename T2, typename spglue_type> inline SpMat&  operator=(const mtSpGlue<eT, T1, T2, spglue_type>& X);
+  template<typename T1, typename T2, typename spglue_type> inline SpMat& operator+=(const mtSpGlue<eT, T1, T2, spglue_type>& X);
+  template<typename T1, typename T2, typename spglue_type> inline SpMat& operator-=(const mtSpGlue<eT, T1, T2, spglue_type>& X);
+  template<typename T1, typename T2, typename spglue_type> inline SpMat& operator*=(const mtSpGlue<eT, T1, T2, spglue_type>& X);
+  template<typename T1, typename T2, typename spglue_type> inline SpMat& operator%=(const mtSpGlue<eT, T1, T2, spglue_type>& X);
+  template<typename T1, typename T2, typename spglue_type> inline SpMat& operator/=(const mtSpGlue<eT, T1, T2, spglue_type>& X);
+  
   
   arma_inline       SpSubview<eT> row(const uword row_num);
   arma_inline const SpSubview<eT> row(const uword row_num) const;
@@ -242,18 +262,18 @@ class SpMat : public SpBase< eT, SpMat<eT> >
   
   
   // access the i-th element; if there is nothing at element i, 0 is returned
-  arma_inline arma_warn_unused MapMat_elem<eT> operator[] (const uword i);
-  arma_inline arma_warn_unused eT              operator[] (const uword i) const;
-  arma_inline arma_warn_unused MapMat_elem<eT> at         (const uword i);
-  arma_inline arma_warn_unused eT              at         (const uword i) const;
-  arma_inline arma_warn_unused MapMat_elem<eT> operator() (const uword i);
-  arma_inline arma_warn_unused eT              operator() (const uword i) const;
+  arma_inline arma_warn_unused SpMat_MapMat_val<eT> operator[] (const uword i);
+  arma_inline arma_warn_unused eT                   operator[] (const uword i) const;
+  arma_inline arma_warn_unused SpMat_MapMat_val<eT> at         (const uword i);
+  arma_inline arma_warn_unused eT                   at         (const uword i) const;
+  arma_inline arma_warn_unused SpMat_MapMat_val<eT> operator() (const uword i);
+  arma_inline arma_warn_unused eT                   operator() (const uword i) const;
   
   // access the element at the given row and column; if there is nothing at that position, 0 is returned
-  arma_inline arma_warn_unused MapMat_elem<eT> at         (const uword in_row, const uword in_col);
-  arma_inline arma_warn_unused eT              at         (const uword in_row, const uword in_col) const;
-  arma_inline arma_warn_unused MapMat_elem<eT> operator() (const uword in_row, const uword in_col);
-  arma_inline arma_warn_unused eT              operator() (const uword in_row, const uword in_col) const;
+  arma_inline arma_warn_unused SpMat_MapMat_val<eT> at         (const uword in_row, const uword in_col);
+  arma_inline arma_warn_unused eT                   at         (const uword in_row, const uword in_col) const;
+  arma_inline arma_warn_unused SpMat_MapMat_val<eT> operator() (const uword in_row, const uword in_col);
+  arma_inline arma_warn_unused eT                   operator() (const uword in_row, const uword in_col) const;
   
   
   arma_inline arma_warn_unused bool is_empty()  const;
@@ -263,6 +283,12 @@ class SpMat : public SpBase< eT, SpMat<eT> >
   arma_inline arma_warn_unused bool is_square() const;
        inline arma_warn_unused bool is_finite() const;
   
+  inline arma_warn_unused bool is_symmetric() const;
+  inline arma_warn_unused bool is_symmetric(const typename get_pod_type<eT>::result tol) const;
+  
+  inline arma_warn_unused bool is_hermitian() const;
+  inline arma_warn_unused bool is_hermitian(const typename get_pod_type<eT>::result tol) const;
+  
   inline arma_warn_unused bool has_inf() const;
   inline arma_warn_unused bool has_nan() const;
   
@@ -277,17 +303,17 @@ class SpMat : public SpBase< eT, SpMat<eT> >
   arma_inline arma_warn_unused bool in_range(const uword in_row, const uword in_col, const SizeMat& s) const;
   
   
-  inline void impl_print(                           const std::string& extra_text) const;
-  inline void impl_print(std::ostream& user_stream, const std::string& extra_text) const;
+  arma_cold inline void impl_print(                           const std::string& extra_text) const;
+  arma_cold inline void impl_print(std::ostream& user_stream, const std::string& extra_text) const;
   
-  inline void impl_raw_print(                           const std::string& extra_text) const;
-  inline void impl_raw_print(std::ostream& user_stream, const std::string& extra_text) const;
+  arma_cold inline void impl_raw_print(                           const std::string& extra_text) const;
+  arma_cold inline void impl_raw_print(std::ostream& user_stream, const std::string& extra_text) const;
   
-  inline void impl_print_dense(                           const std::string& extra_text) const;
-  inline void impl_print_dense(std::ostream& user_stream, const std::string& extra_text) const;
+  arma_cold inline void impl_print_dense(                           const std::string& extra_text) const;
+  arma_cold inline void impl_print_dense(std::ostream& user_stream, const std::string& extra_text) const;
   
-  inline void impl_raw_print_dense(                           const std::string& extra_text) const;
-  inline void impl_raw_print_dense(std::ostream& user_stream, const std::string& extra_text) const;
+  arma_cold inline void impl_raw_print_dense(                           const std::string& extra_text) const;
+  arma_cold inline void impl_raw_print_dense(std::ostream& user_stream, const std::string& extra_text) const;
   
   
   template<typename eT2> inline void copy_size(const SpMat<eT2>& m);
@@ -302,10 +328,21 @@ class SpMat : public SpBase< eT, SpMat<eT> >
   
   inline void  reshape(const uword in_rows, const uword in_cols);
   inline void  reshape(const SizeMat& s);
-  inline void  reshape(const uword in_rows, const uword in_cols, const uword dim);  // this form is deprecated: don't use it
+  
+  inline void  reshape_helper_generic(const uword in_rows, const uword in_cols);  //! internal use only
+  inline void  reshape_helper_intovec();                                          //! internal use only
+  
+  arma_deprecated inline void reshape(const uword in_rows, const uword in_cols, const uword dim);  //!< NOTE: don't use this form: it will be removed
+  
+  template<typename functor> inline const SpMat&  for_each(functor F);
+  template<typename functor> inline const SpMat&  for_each(functor F) const;
+  
+  template<typename functor> inline const SpMat& transform(functor F);
   
   inline const SpMat& replace(const eT old_val, const eT new_val);
   
+  inline const SpMat& clean(const pod_type threshold);
+  
   inline const SpMat& zeros();
   inline const SpMat& zeros(const uword in_elem);
   inline const SpMat& zeros(const uword in_rows, const uword in_cols);
@@ -327,6 +364,15 @@ class SpMat : public SpBase< eT, SpMat<eT> >
   
   inline void reset();
   
+  //! don't use this unless you're writing internal Armadillo code
+  inline void reserve(const uword in_rows, const uword in_cols, const uword new_n_nonzero);
+  
+  //! don't use this unless you're writing internal Armadillo code
+  inline SpMat(const arma_reserve_indicator&, const uword in_rows, const uword in_cols, const uword new_n_nonzero);
+  
+  //! don't use this unless you're writing internal Armadillo code
+  template<typename eT2>
+  inline SpMat(const arma_layout_indicator&, const SpMat<eT2>& x);
   
   template<typename T1> inline void set_real(const SpBase<pod_type,T1>& X);
   template<typename T1> inline void set_imag(const SpBase<pod_type,T1>& X);
@@ -335,17 +381,21 @@ class SpMat : public SpBase< eT, SpMat<eT> >
   // saving and loading
   // TODO: implement auto_detect for sparse matrices
   
-  inline bool save(const std::string   name, const file_type type = arma_binary, const bool print_status = true) const;
-  inline bool save(      std::ostream& os,   const file_type type = arma_binary, const bool print_status = true) const;
+  inline arma_cold bool save(const std::string   name, const file_type type = arma_binary, const bool print_status = true) const;
+  inline arma_cold bool save(const csv_name&     spec, const file_type type =   csv_ascii, const bool print_status = true) const;
+  inline arma_cold bool save(      std::ostream& os,   const file_type type = arma_binary, const bool print_status = true) const;
   
-  inline bool load(const std::string   name, const file_type type = arma_binary, const bool print_status = true);
-  inline bool load(      std::istream& is,   const file_type type = arma_binary, const bool print_status = true);
+  inline arma_cold bool load(const std::string   name, const file_type type = arma_binary, const bool print_status = true);
+  inline arma_cold bool load(const csv_name&     spec, const file_type type =   csv_ascii, const bool print_status = true);
+  inline arma_cold bool load(      std::istream& is,   const file_type type = arma_binary, const bool print_status = true);
   
-  inline bool quiet_save(const std::string   name, const file_type type = arma_binary) const;
-  inline bool quiet_save(      std::ostream& os,   const file_type type = arma_binary) const;
+  inline arma_cold bool quiet_save(const std::string   name, const file_type type = arma_binary) const;
+  inline arma_cold bool quiet_save(const csv_name&     spec, const file_type type =   csv_ascii) const;
+  inline arma_cold bool quiet_save(      std::ostream& os,   const file_type type = arma_binary) const;
   
-  inline bool quiet_load(const std::string   name, const file_type type = arma_binary);
-  inline bool quiet_load(      std::istream& is,   const file_type type = arma_binary);
+  inline arma_cold bool quiet_load(const std::string   name, const file_type type = arma_binary);
+  inline arma_cold bool quiet_load(const csv_name&     spec, const file_type type =   csv_ascii);
+  inline arma_cold bool quiet_load(      std::istream& is,   const file_type type = arma_binary);
   
   
   
@@ -376,10 +426,9 @@ class SpMat : public SpBase< eT, SpMat<eT> >
     arma_aligned       uword  internal_col;
     arma_aligned       uword  internal_pos;
     
-    // so that we satisfy the STL iterator types
     typedef std::bidirectional_iterator_tag iterator_category;
     typedef eT                              value_type;
-    typedef uword                           difference_type; // not certain on this one
+    typedef std::ptrdiff_t                  difference_type;  // TODO: not certain on this one
     typedef const eT*                       pointer;
     typedef const eT&                       reference;
     };
@@ -396,11 +445,11 @@ class SpMat : public SpBase< eT, SpMat<eT> >
     inline const_iterator(const SpMat& in_M, uword in_row, uword in_col, uword in_pos);
     inline const_iterator(const const_iterator& other);
     
-    inline arma_hot const_iterator& operator++();
-    inline arma_hot const_iterator  operator++(int);
+    inline arma_hot         const_iterator& operator++();
+    inline arma_warn_unused const_iterator  operator++(int);
     
-    inline arma_hot const_iterator& operator--();
-    inline arma_hot const_iterator  operator--(int);
+    inline arma_hot         const_iterator& operator--();
+    inline arma_warn_unused const_iterator  operator--(int);
     
     inline arma_hot bool operator==(const const_iterator& rhs) const;
     inline arma_hot bool operator!=(const const_iterator& rhs) const;
@@ -433,11 +482,11 @@ class SpMat : public SpBase< eT, SpMat<eT> >
     inline arma_hot SpValProxy<SpMat<eT> > operator*();
     
     // overloads needed for return type correctness
-    inline arma_hot iterator& operator++();
-    inline arma_hot iterator  operator++(int);
+    inline arma_hot         iterator& operator++();
+    inline arma_warn_unused iterator  operator++(int);
     
-    inline arma_hot iterator& operator--();
-    inline arma_hot iterator  operator--(int);
+    inline arma_hot         iterator& operator--();
+    inline arma_warn_unused iterator  operator--(int);
     
     // this has a different value_type than iterator_base
     typedef SpValProxy<SpMat<eT> >         value_type;
@@ -455,14 +504,14 @@ class SpMat : public SpBase< eT, SpMat<eT> >
     inline const_row_iterator(const SpMat& in_M, uword in_row, uword in_col);
     inline const_row_iterator(const const_row_iterator& other);
     
-    inline arma_hot const_row_iterator& operator++();
-    inline arma_hot const_row_iterator  operator++(int);
+    inline arma_hot         const_row_iterator& operator++();
+    inline arma_warn_unused const_row_iterator  operator++(int);
     
-    inline arma_hot const_row_iterator& operator--();
-    inline arma_hot const_row_iterator  operator--(int);
+    inline arma_hot         const_row_iterator& operator--();
+    inline arma_warn_unused const_row_iterator  operator--(int);
     
-    uword internal_row; // hold row internally because we use internal_pos differently
-    uword actual_pos;   // actual position in matrix
+    uword internal_row; // hold row internally
+    uword actual_pos; // this holds the true position we are at in the matrix, as column-major indexing
     
     arma_inline eT operator*() const { return iterator_base::M->values[actual_pos]; }
     
@@ -494,11 +543,11 @@ class SpMat : public SpBase< eT, SpMat<eT> >
     inline arma_hot SpValProxy<SpMat<eT> > operator*();
     
     // overloads required for return type correctness
-    inline arma_hot row_iterator& operator++();
-    inline arma_hot row_iterator  operator++(int);
+    inline arma_hot         row_iterator& operator++();
+    inline arma_warn_unused row_iterator  operator++(int);
     
-    inline arma_hot row_iterator& operator--();
-    inline arma_hot row_iterator  operator--(int);
+    inline arma_hot         row_iterator& operator--();
+    inline arma_warn_unused row_iterator  operator--(int);
     
     // this has a different value_type than iterator_base
     typedef SpValProxy<SpMat<eT> >         value_type;
@@ -507,21 +556,32 @@ class SpMat : public SpBase< eT, SpMat<eT> >
     };
   
   
+  typedef       iterator       col_iterator;
+  typedef const_iterator const_col_iterator;
+  
   typedef       iterator       row_col_iterator;
   typedef const_iterator const_row_col_iterator;
   
   
   inline       iterator     begin();
   inline const_iterator     begin() const;
+  inline const_iterator    cbegin() const;
   
   inline       iterator     end();
   inline const_iterator     end() const;
+  inline const_iterator    cend() const;
   
-  inline       iterator     begin_col(const uword col_num);
-  inline const_iterator     begin_col(const uword col_num) const;
+  inline       col_iterator begin_col(const uword col_num);
+  inline const_col_iterator begin_col(const uword col_num) const;
   
-  inline       iterator     end_col(const uword col_num);
-  inline const_iterator     end_col(const uword col_num) const;
+  inline       col_iterator begin_col_no_sync(const uword col_num);
+  inline const_col_iterator begin_col_no_sync(const uword col_num) const;
+  
+  inline       col_iterator end_col(const uword col_num);
+  inline const_col_iterator end_col(const uword col_num) const;
+  
+  inline       col_iterator end_col_no_sync(const uword col_num);
+  inline const_col_iterator end_col_no_sync(const uword col_num) const;
   
   inline       row_iterator begin_row(const uword row_num = 0);
   inline const_row_iterator begin_row(const uword row_num = 0) const;
@@ -565,14 +625,20 @@ class SpMat : public SpBase< eT, SpMat<eT> >
   template<              typename T1, typename Functor> arma_hot inline void init_xform   (const SpBase<eT, T1>& x, const Functor& func);
   template<typename eT2, typename T1, typename Functor> arma_hot inline void init_xform_mt(const SpBase<eT2,T1>& x, const Functor& func);
   
+  //! don't use this unless you're writing internal Armadillo code
+  arma_inline bool is_alias(const SpMat<eT>& X) const;
+  
   
   protected:
   
-  inline void init(uword in_rows, uword in_cols);
+  inline void                init(uword in_rows, uword in_cols, const uword new_n_nonzero = 0);
+  inline void arma_cold init_cold(uword in_rows, uword in_cols, const uword new_n_nonzero = 0);
+  
   inline void init(const std::string& text);
   inline void init(const  SpMat<eT>& x);
   inline void init(const MapMat<eT>& x);
   
+  inline void init_simple(const SpMat<eT>& x);
   
   inline void init_batch_std(const Mat<uword>& locations, const Mat<eT>& values, const bool sort_locations);
   inline void init_batch_add(const Mat<uword>& locations, const Mat<eT>& values, const bool sort_locations);
@@ -583,16 +649,19 @@ class SpMat : public SpBase< eT, SpMat<eT> >
   
   private:
   
-  inline arma_hot arma_warn_unused SpValProxy<SpMat<eT> > get_value(const uword i);
-  inline arma_hot arma_warn_unused eT                     get_value(const uword i) const;
+  inline arma_hot arma_warn_unused const eT* find_value_csc(const uword in_row, const uword in_col) const;
   
-  inline arma_hot arma_warn_unused SpValProxy<SpMat<eT> > get_value(const uword in_row, const uword in_col);
-  inline arma_hot arma_warn_unused eT                     get_value(const uword in_row, const uword in_col) const;
+  inline arma_hot arma_warn_unused eT get_value(const uword i                         ) const;
+  inline arma_hot arma_warn_unused eT get_value(const uword in_row, const uword in_col) const;
   
+  inline arma_hot arma_warn_unused eT get_value_csc(const uword i                         ) const;
+  inline arma_hot arma_warn_unused eT get_value_csc(const uword in_row, const uword in_col) const;
   
-  arma_inline arma_hot arma_warn_unused uword get_position(const uword i) const;
-  arma_inline arma_hot                  void  get_position(const uword i, uword& row_of_i, uword& col_of_i) const;
-  
+  inline arma_hot arma_warn_unused bool try_set_value_csc(const uword in_row, const uword in_col, const eT in_val);
+  inline arma_hot arma_warn_unused bool try_add_value_csc(const uword in_row, const uword in_col, const eT in_val);
+  inline arma_hot arma_warn_unused bool try_sub_value_csc(const uword in_row, const uword in_col, const eT in_val);
+  inline arma_hot arma_warn_unused bool try_mul_value_csc(const uword in_row, const uword in_col, const eT in_val);
+  inline arma_hot arma_warn_unused bool try_div_value_csc(const uword in_row, const uword in_col, const eT in_val);
   
   inline arma_warn_unused eT&  insert_element(const uword in_row, const uword in_col, const eT in_val = eT(0));
   inline                  void delete_element(const uword in_row, const uword in_col);
@@ -602,29 +671,34 @@ class SpMat : public SpBase< eT, SpMat<eT> >
   
   arma_aligned mutable MapMat<eT> cache;
   arma_aligned mutable state_type sync_state;
-  // 0: cache needs to be updated from CSC
-  // 1: CSC needs to be updated from cache
-  // 2: no update required
+  // 0: cache needs to be updated from CSC (ie.   CSC has more recent data)
+  // 1: CSC needs to be updated from cache (ie. cache has more recent data)
+  // 2: no update required                 (ie. CSC and cache contain the same data)
   
-  #if !defined(_OPENMP) && defined(ARMA_USE_CXX11)
+  #if defined(ARMA_USE_CXX11)
   arma_aligned mutable std::mutex cache_mutex;
   #endif
   
   arma_inline void invalidate_cache() const;
   arma_inline void invalidate_csc()   const;
   
-  inline void sync_cache() const;
-  inline void sync_csc()   const;
+  inline void sync_cache()        const;
+  inline void sync_cache_simple() const;
+  inline void sync_csc()          const;
+  inline void sync_csc_simple()   const;
   
   
   friend class SpValProxy< SpMat<eT> >;  // allow SpValProxy to call insert_element() and delete_element()
   friend class SpSubview<eT>;
   friend class SpRow<eT>;
   friend class SpCol<eT>;
+  friend class SpMat_MapMat_val<eT>;
+  friend class SpSubview_MapMat_val<eT>;
+  friend class spdiagview<eT>;
   
   
   public:
-    
+  
   #ifdef ARMA_EXTRA_SPMAT_PROTO
     #include ARMA_INCFILE_WRAP(ARMA_EXTRA_SPMAT_PROTO)
   #endif
