@@ -1,11 +1,17 @@
-// Copyright (C) 2008-2013 National ICT Australia (NICTA)
+// Copyright 2008-2016 Conrad Sanderson (http://conradsanderson.id.au)
+// Copyright 2008-2016 National ICT Australia (NICTA)
 // 
-// This Source Code Form is subject to the terms of the Mozilla Public
-// License, v. 2.0. If a copy of the MPL was not distributed with this
-// file, You can obtain one at http://mozilla.org/MPL/2.0/.
-// -------------------------------------------------------------------
+// Licensed under the Apache License, Version 2.0 (the "License");
+// you may not use this file except in compliance with the License.
+// You may obtain a copy of the License at
+// http://www.apache.org/licenses/LICENSE-2.0
 // 
-// Written by Conrad Sanderson - http://conradsanderson.id.au
+// Unless required by applicable law or agreed to in writing, software
+// distributed under the License is distributed on an "AS IS" BASIS,
+// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+// See the License for the specific language governing permissions and
+// limitations under the License.
+// ------------------------------------------------------------------------
 
 
 //! \addtogroup op_cx_scalar
@@ -36,7 +42,7 @@ op_cx_scalar_times::apply
   const eT  k       = X.aux_out_eT;
         eT* out_mem = out.memptr();
   
-  if(Proxy<T1>::prefer_at_accessor == false)
+  if(Proxy<T1>::use_at == false)
     {
     const uword n_elem = A.get_n_elem();
   
@@ -80,7 +86,7 @@ op_cx_scalar_plus::apply
   const eT  k       = X.aux_out_eT;
         eT* out_mem = out.memptr();
   
-  if(Proxy<T1>::prefer_at_accessor == false)
+  if(Proxy<T1>::use_at == false)
     {
     const uword n_elem = A.get_n_elem();
   
@@ -124,7 +130,7 @@ op_cx_scalar_minus_pre::apply
   const eT  k       = X.aux_out_eT;
         eT* out_mem = out.memptr();
   
-  if(Proxy<T1>::prefer_at_accessor == false)
+  if(Proxy<T1>::use_at == false)
     {
     const uword n_elem = A.get_n_elem();
   
@@ -168,7 +174,7 @@ op_cx_scalar_minus_post::apply
   const eT  k       = X.aux_out_eT;
         eT* out_mem = out.memptr();
   
-  if(Proxy<T1>::prefer_at_accessor == false)
+  if(Proxy<T1>::use_at == false)
     {
     const uword n_elem = A.get_n_elem();
   
@@ -212,7 +218,7 @@ op_cx_scalar_div_pre::apply
   const eT  k       = X.aux_out_eT;
         eT* out_mem = out.memptr();
   
-  if(Proxy<T1>::prefer_at_accessor == false)
+  if(Proxy<T1>::use_at == false)
     {
     const uword n_elem = A.get_n_elem();
   
@@ -256,7 +262,7 @@ op_cx_scalar_div_post::apply
   const eT  k       = X.aux_out_eT;
         eT* out_mem = out.memptr();
   
-  if(Proxy<T1>::prefer_at_accessor == false)
+  if(Proxy<T1>::use_at == false)
     {
     const uword n_elem = A.get_n_elem();
   
@@ -308,7 +314,7 @@ op_cx_scalar_times::apply
   const uword n_elem  = out.n_elem;
         eT*   out_mem = out.memptr();
   
-  if(ProxyCube<T1>::prefer_at_accessor == false)
+  if(ProxyCube<T1>::use_at == false)
     {
     for(uword i=0; i<n_elem; ++i)
       {
@@ -353,7 +359,7 @@ op_cx_scalar_plus::apply
   const uword n_elem  = out.n_elem;
         eT*   out_mem = out.memptr();
   
-  if(ProxyCube<T1>::prefer_at_accessor == false)
+  if(ProxyCube<T1>::use_at == false)
     {
     for(uword i=0; i<n_elem; ++i)
       {
@@ -398,7 +404,7 @@ op_cx_scalar_minus_pre::apply
   const uword n_elem  = out.n_elem;
         eT*   out_mem = out.memptr();
   
-  if(ProxyCube<T1>::prefer_at_accessor == false)
+  if(ProxyCube<T1>::use_at == false)
     {
     for(uword i=0; i<n_elem; ++i)
       {
@@ -443,7 +449,7 @@ op_cx_scalar_minus_post::apply
   const uword n_elem  = out.n_elem;
         eT*   out_mem = out.memptr();
   
-  if(ProxyCube<T1>::prefer_at_accessor == false)
+  if(ProxyCube<T1>::use_at == false)
     {
     for(uword i=0; i<n_elem; ++i)
       {
@@ -488,7 +494,7 @@ op_cx_scalar_div_pre::apply
   const uword n_elem  = out.n_elem;
         eT*   out_mem = out.memptr();
   
-  if(ProxyCube<T1>::prefer_at_accessor == false)
+  if(ProxyCube<T1>::use_at == false)
     {
     for(uword i=0; i<n_elem; ++i)
       {
@@ -533,7 +539,7 @@ op_cx_scalar_div_post::apply
   const uword n_elem  = out.n_elem;
         eT*   out_mem = out.memptr();
   
-  if(ProxyCube<T1>::prefer_at_accessor == false)
+  if(ProxyCube<T1>::use_at == false)
     {
     for(uword i=0; i<n_elem; ++i)
       {
