@@ -19,6 +19,7 @@
 
 
 class op_index_max
+  : public traits_op_xvec
   {
   public:
   
@@ -31,6 +32,18 @@ class op_index_max
   inline static void apply_noalias(Mat<uword>& out, const Mat<eT>& X, const uword dim);
   
   
+  // cubes
+  
+  template<typename T1>
+  inline static void apply(Cube<uword>& out, const mtOpCube<uword, T1, op_index_max>& in);  
+  
+  template<typename eT>
+  inline static void apply_noalias(Cube<uword>& out, const Cube<eT>& X, const uword dim, const typename arma_not_cx<eT>::result* junk = 0);
+  
+  template<typename eT>
+  inline static void apply_noalias(Cube<uword>& out, const Cube<eT>& X, const uword dim, const typename arma_cx_only<eT>::result* junk = 0);
+  
+  
   // sparse matrices
   
   template<typename T1>
